@@ -16,10 +16,10 @@
 import json
 import posixpath
 
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log as oslo_logging
 
 from cloudbaseinit.metadata.services import base
-from cloudbaseinit.openstack.common import log as logging
 from cloudbaseinit.utils import debiface
 from cloudbaseinit.utils import encoding
 from cloudbaseinit.utils import x509constants
@@ -33,7 +33,7 @@ opts = [
 CONF = cfg.CONF
 CONF.register_opts(opts)
 
-LOG = logging.getLogger(__name__)
+LOG = oslo_logging.getLogger(__name__)
 
 
 class BaseOpenStackService(base.BaseMetadataService):
@@ -51,9 +51,9 @@ class BaseOpenStackService(base.BaseMetadataService):
     def _get_meta_data(self, version='latest'):
         path = posixpath.normpath(
             posixpath.join('openstack', version, 'meta_data.json'))
-        data = self._get_cache_data(path)
+        data = self._get_cache_data(path, decode=True)
         if data:
-            return json.loads(encoding.get_as_string(data))
+            return json.loads(data)
 
     def get_instance_id(self):
         return self._get_meta_data().get('uuid')
@@ -136,10 +136,10 @@ class BaseOpenStackService(base.BaseMetadataService):
         if not certs:
             # Look if the user_data contains a PEM certificate
             try:
-                user_data = self.get_user_data()
+                user_data = self.get_user_data().strip()
                 if user_data.startswith(
                         x509constants.PEM_HEADER.encode()):
-                    certs.append(user_data)
+                    certs.append(encoding.get_as_string(user_data))
             except base.NotExistingMetadataException:
                 LOG.debug("user_data metadata not present")
 
