@@ -12,7 +12,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-
+import contextlib
 import ctypes
 from ctypes import wintypes
 import os
@@ -20,25 +20,26 @@ import re
 import struct
 import time
 
-import pywintypes
+from oslo_log import log as oslo_logging
 import six
 from six.moves import winreg
 from tzlocal import windows_tz
 from win32com import client
+import win32net
+import win32netcon
 import win32process
 import win32security
 import wmi
 
 from cloudbaseinit import exception
-from cloudbaseinit.openstack.common import log as logging
 from cloudbaseinit.osutils import base
-from cloudbaseinit.utils import encoding
+from cloudbaseinit.utils.windows import disk
 from cloudbaseinit.utils.windows import network
 from cloudbaseinit.utils.windows import privilege
 from cloudbaseinit.utils.windows import timezone
 
 
-LOG = logging.getLogger(__name__)
+LOG = oslo_logging.getLogger(__name__)
 AF_INET6 = 23
 UNICAST = 1
 MANUAL = 1
@@ -52,6 +53,7 @@ iphlpapi = ctypes.windll.iphlpapi
 Ws2_32 = ctypes.windll.Ws2_32
 setupapi = ctypes.windll.setupapi
 msvcrt = ctypes.cdll.msvcrt
+ntdll = ctypes.windll.ntdll
 
 
 class Win32_PROFILEINFO(ctypes.Structure):
@@ -107,39 +109,18 @@ class Win32_OSVERSIONINFOEX_W(ctypes.Structure):
         ('dwBuildNumber', wintypes.DWORD),
         ('dwPlatformId', wintypes.DWORD),
         ('szCSDVersion', wintypes.WCHAR * 128),
-        ('wServicePackMajor', wintypes.DWORD),
-        ('wServicePackMinor', wintypes.DWORD),
-        ('wSuiteMask', wintypes.DWORD),
+        ('wServicePackMajor', wintypes.WORD),
+        ('wServicePackMinor', wintypes.WORD),
+        ('wSuiteMask', wintypes.WORD),
         ('wProductType', wintypes.BYTE),
         ('wReserved', wintypes.BYTE)
     ]
 
 
-class GUID(ctypes.Structure):
-    _fields_ = [
-        ("data1", ctypes.wintypes.DWORD),
-        ("data2", ctypes.wintypes.WORD),
-        ("data3", ctypes.wintypes.WORD),
-        ("data4", ctypes.c_byte * 8)]
-
-    def __init__(self, l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8):
-        self.data1 = l
-        self.data2 = w1
-        self.data3 = w2
-        self.data4[0] = b1
-        self.data4[1] = b2
-        self.data4[2] = b3
-        self.data4[3] = b4
-        self.data4[4] = b5
-        self.data4[5] = b6
-        self.data4[6] = b7
-        self.data4[7] = b8
-
-
 class Win32_SP_DEVICE_INTERFACE_DATA(ctypes.Structure):
     _fields_ = [
         ('cbSize', wintypes.DWORD),
-        ('InterfaceClassGuid', GUID),
+        ('InterfaceClassGuid', disk.GUID),
         ('Flags', wintypes.DWORD),
         ('Reserved', ctypes.POINTER(wintypes.ULONG))
     ]
@@ -166,10 +147,10 @@ msvcrt.malloc.restype = ctypes.c_void_p
 msvcrt.free.argtypes = [ctypes.c_void_p]
 msvcrt.free.restype = None
 
-kernel32.VerifyVersionInfoW.argtypes = [
+ntdll.RtlVerifyVersionInfo.argtypes = [
     ctypes.POINTER(Win32_OSVERSIONINFOEX_W),
     wintypes.DWORD, wintypes.ULARGE_INTEGER]
-kernel32.VerifyVersionInfoW.restype = wintypes.BOOL
+ntdll.RtlVerifyVersionInfo.restype = wintypes.DWORD
 
 kernel32.VerSetConditionMask.argtypes = [wintypes.ULARGE_INTEGER,
                                          wintypes.DWORD,
@@ -217,7 +198,7 @@ iphlpapi.GetIpForwardTable.restype = wintypes.DWORD
 
 Ws2_32.inet_ntoa.restype = ctypes.c_char_p
 
-setupapi.SetupDiGetClassDevsW.argtypes = [ctypes.POINTER(GUID),
+setupapi.SetupDiGetClassDevsW.argtypes = [ctypes.POINTER(disk.GUID),
                                           wintypes.LPCWSTR,
                                           wintypes.HANDLE,
                                           wintypes.DWORD]
@@ -226,7 +207,7 @@ setupapi.SetupDiGetClassDevsW.restype = wintypes.HANDLE
 setupapi.SetupDiEnumDeviceInterfaces.argtypes = [
     wintypes.HANDLE,
     wintypes.LPVOID,
-    ctypes.POINTER(GUID),
+    ctypes.POINTER(disk.GUID),
     wintypes.DWORD,
     ctypes.POINTER(Win32_SP_DEVICE_INTERFACE_DATA)]
 setupapi.SetupDiEnumDeviceInterfaces.restype = wintypes.BOOL
@@ -249,21 +230,26 @@ VER_BUILDNUMBER = 4
 
 VER_GREATER_EQUAL = 3
 
-GUID_DEVINTERFACE_DISK = GUID(0x53f56307, 0xb6bf, 0x11d0, 0x94, 0xf2,
-                              0x00, 0xa0, 0xc9, 0x1e, 0xfb, 0x8b)
+GUID_DEVINTERFACE_DISK = disk.GUID(0x53f56307, 0xb6bf, 0x11d0, 0x94, 0xf2,
+                                   0x00, 0xa0, 0xc9, 0x1e, 0xfb, 0x8b)
 
 
 class WindowsUtils(base.BaseOSUtils):
     NERR_GroupNotFound = 2220
+    NERR_UserNotFound = 2221
     ERROR_ACCESS_DENIED = 5
     ERROR_INSUFFICIENT_BUFFER = 122
     ERROR_NO_DATA = 232
     ERROR_NO_SUCH_MEMBER = 1387
     ERROR_MEMBER_IN_ALIAS = 1378
     ERROR_INVALID_MEMBER = 1388
-    ERROR_OLD_WIN_VERSION = 1150
     ERROR_NO_MORE_FILES = 18
 
+    STATUS_REVISION_MISMATCH = 0xC0000059
+
+    ADS_UF_PASSWORD_EXPIRED = 0x800000
+    PASSWORD_CHANGED_FLAG = 1
+
     INVALID_HANDLE_VALUE = 0xFFFFFFFF
 
     FILE_SHARE_READ = 1
@@ -305,82 +291,87 @@ class WindowsUtils(base.BaseOSUtils):
 
     def reboot(self):
         with privilege.acquire_privilege(win32security.SE_SHUTDOWN_NAME):
-            ret_val = advapi32.InitiateSystemShutdownW(
+            ret_val = advapi32.InitiateSystemShutdownExW(
                 0, "Cloudbase-Init reboot",
-                0, True, True)
+                0, True, True, 0)
             if not ret_val:
                 raise exception.WindowsCloudbaseInitException(
                     "Reboot failed: %r")
 
-    def _get_user_wmi_object(self, username):
-        conn = wmi.WMI(moniker='//./root/cimv2')
-        username_san = self._sanitize_wmi_input(username)
-        q = conn.query('SELECT * FROM Win32_Account where name = '
-                       '\'%s\'' % username_san)
-        if len(q) > 0:
-            return q[0]
-        return None
-
     def user_exists(self, username):
-        return self._get_user_wmi_object(username) is not None
-
-    def _get_adsi_object(self, hostname='.', object_name=None,
-                         object_type='computer'):
-        adsi = client.Dispatch("ADsNameSpaces")
-        winnt = adsi.GetObject("", "WinNT:")
-        query = "WinNT://%s" % hostname
-        if object_name:
-            object_name_san = self.sanitize_shell_input(object_name)
-            query += "/%s" % object_name_san
-        query += ",%s" % object_type
-        return winnt.OpenDSObject(query, "", "", 0)
-
-    def _create_or_change_user(self, username, password, create,
-                               password_expires):
         try:
-            if create:
-                host = self._get_adsi_object()
-                user = host.Create('user', username)
-            else:
-                user = self._get_adsi_object(object_name=username,
-                                             object_type='user')
-
-            user.setpassword(password)
-            user.SetInfo()
-
-            self._set_user_password_expiration(username, password_expires)
-        except pywintypes.com_error as ex:
-            if create:
-                msg = "Create user failed: %s"
-            else:
-                msg = "Set user password failed: %s"
-            raise exception.CloudbaseInitException(msg % ex.excepinfo[2])
-
-    def _sanitize_wmi_input(self, value):
-        return value.replace('\'', '\'\'')
-
-    def _set_user_password_expiration(self, username, password_expires):
-        r = self._get_user_wmi_object(username)
-        if not r:
+            self._get_user_info(username, 1)
+            return True
+        except exception.ItemNotFoundException:
+            # User not found
             return False
-        r.PasswordExpires = password_expires
-        r.Put_()
-        return True
 
     def create_user(self, username, password, password_expires=False):
-        self._create_or_change_user(username, password, create=True,
-                                    password_expires=password_expires)
+        user_info = {
+            "name": username,
+            "password": password,
+            "priv": win32netcon.USER_PRIV_USER,
+            "flags": win32netcon.UF_NORMAL_ACCOUNT | win32netcon.UF_SCRIPT,
+        }
+
+        if not password_expires:
+            user_info["flags"] |= win32netcon.UF_DONT_EXPIRE_PASSWD
+
+        try:
+            win32net.NetUserAdd(None, 1, user_info)
+        except win32net.error as ex:
+            raise exception.CloudbaseInitException(
+                "Create user failed: %s" % ex.args[2])
+
+    def _get_user_info(self, username, level):
+        try:
+            return win32net.NetUserGetInfo(None, username, level)
+        except win32net.error as ex:
+            if ex.args[0] == self.NERR_UserNotFound:
+                raise exception.ItemNotFoundException(
+                    "User not found: %s" % username)
+            else:
+                raise exception.CloudbaseInitException(
+                    "Failed to get user info: %s" % ex.args[2])
 
     def set_user_password(self, username, password, password_expires=False):
-        self._create_or_change_user(username, password, create=False,
-                                    password_expires=password_expires)
+        user_info = self._get_user_info(username, 1)
+        user_info["password"] = password
+
+        if password_expires:
+            user_info["flags"] &= ~win32netcon.UF_DONT_EXPIRE_PASSWD
+        else:
+            user_info["flags"] |= win32netcon.UF_DONT_EXPIRE_PASSWD
+
+        try:
+            win32net.NetUserSetInfo(None, username, 1, user_info)
+        except win32net.error as ex:
+            raise exception.CloudbaseInitException(
+                "Set user password failed: %s" % ex.args[2])
+
+    def change_password_next_logon(self, username):
+        """Force the given user to change the password at next logon."""
+        user_info = self._get_user_info(username, 4)
+        user_info["flags"] &= ~win32netcon.UF_DONT_EXPIRE_PASSWD
+        user_info["password_expired"] = 1
+
+        try:
+            win32net.NetUserSetInfo(None, username, 4, user_info)
+        except win32net.error as ex:
+            raise exception.CloudbaseInitException(
+                "Setting password expiration failed: %s" % ex.args[2])
+
+    @staticmethod
+    def _get_cch_referenced_domain_name(domain_name):
+        return wintypes.DWORD(
+            ctypes.sizeof(domain_name) // ctypes.sizeof(wintypes.WCHAR))
 
     def _get_user_sid_and_domain(self, username):
         sid = ctypes.create_string_buffer(1024)
         cbSid = wintypes.DWORD(ctypes.sizeof(sid))
         domainName = ctypes.create_unicode_buffer(1024)
-        cchReferencedDomainName = wintypes.DWORD(
-            ctypes.sizeof(domainName) / ctypes.sizeof(wintypes.WCHAR))
+        cchReferencedDomainName = self._get_cch_referenced_domain_name(
+            domainName)
         sidNameUse = wintypes.DWORD()
 
         ret_val = advapi32.LookupAccountNameW(
@@ -390,7 +381,7 @@ class WindowsUtils(base.BaseOSUtils):
             raise exception.WindowsCloudbaseInitException(
                 "Cannot get user SID: %r")
 
-        return (sid, domainName.value)
+        return sid, domainName.value
 
     def add_user_to_local_group(self, username, groupname):
 
@@ -415,10 +406,12 @@ class WindowsUtils(base.BaseOSUtils):
             raise exception.CloudbaseInitException('Unknown error')
 
     def get_user_sid(self, username):
-        r = self._get_user_wmi_object(username)
-        if not r:
-            return None
-        return r.SID
+        try:
+            user_info = self._get_user_info(username, 4)
+            return str(user_info["user_sid"])[6:]
+        except exception.ItemNotFoundException:
+            # User not found
+            pass
 
     def create_user_logon_session(self, username, password, domain='.',
                                   load_profile=True):
@@ -453,7 +446,7 @@ class WindowsUtils(base.BaseOSUtils):
                                 'Microsoft\\Windows NT\\CurrentVersion\\'
                                 'ProfileList\\%s' % user_sid) as key:
                 return winreg.QueryValueEx(key, 'ProfileImagePath')[0]
-        LOG.debug('Home directory not found for user \'%s\'' % username)
+        LOG.debug('Home directory not found for user %r', username)
         return None
 
     def sanitize_shell_input(self, value):
@@ -525,7 +518,7 @@ class WindowsUtils(base.BaseOSUtils):
             iface_index = iface_index_list[0]
 
             LOG.debug('Setting MTU for interface "%(mac_address)s" with '
-                      'value "%(mtu)s"' %
+                      'value "%(mtu)s"',
                       {'mac_address': mac_address, 'mtu': mtu})
 
             base_dir = self._get_system_dir()
@@ -558,7 +551,8 @@ class WindowsUtils(base.BaseOSUtils):
         (ret_val,) = adapter_config.EnableStatic([address], [netmask])
         if ret_val > 1:
             raise exception.CloudbaseInitException(
-                "Cannot set static IP address on network adapter")
+                "Cannot set static IP address on network adapter (%d)",
+                ret_val)
         reboot_required = (ret_val == 1)
 
         if gateway:
@@ -566,7 +560,8 @@ class WindowsUtils(base.BaseOSUtils):
             (ret_val,) = adapter_config.SetGateways([gateway], [1])
             if ret_val > 1:
                 raise exception.CloudbaseInitException(
-                    "Cannot set gateway on network adapter")
+                    "Cannot set gateway on network adapter (%d)",
+                    ret_val)
             reboot_required = reboot_required or ret_val == 1
 
         if dnsnameservers:
@@ -574,7 +569,8 @@ class WindowsUtils(base.BaseOSUtils):
             (ret_val,) = adapter_config.SetDNSServerSearchOrder(dnsnameservers)
             if ret_val > 1:
                 raise exception.CloudbaseInitException(
-                    "Cannot set DNS on network adapter")
+                    "Cannot set DNS on network adapter (%d)",
+                    ret_val)
             reboot_required = reboot_required or ret_val == 1
 
         return reboot_required
@@ -670,7 +666,7 @@ class WindowsUtils(base.BaseOSUtils):
                         break
                     time.sleep(1)
                     LOG.info('Waiting for sysprep completion. '
-                             'GeneralizationState: %d' % gen_state)
+                             'GeneralizationState: %d', gen_state)
         except WindowsError as ex:
             if ex.winerror == 2:
                 LOG.debug('Sysprep data not found in the registry, '
@@ -706,7 +702,7 @@ class WindowsUtils(base.BaseOSUtils):
                                                'ret_val': ret_val})
 
     def start_service(self, service_name):
-        LOG.debug('Starting service %s' % service_name)
+        LOG.debug('Starting service %s', service_name)
         service = self._get_service(service_name)
         (ret_val,) = service.StartService()
         if ret_val != 0:
@@ -716,7 +712,7 @@ class WindowsUtils(base.BaseOSUtils):
                                  'ret_val': ret_val})
 
     def stop_service(self, service_name):
-        LOG.debug('Stopping service %s' % service_name)
+        LOG.debug('Stopping service %s', service_name)
         service = self._get_service(service_name)
         (ret_val,) = service.StopService()
         if ret_val != 0:
@@ -735,66 +731,73 @@ class WindowsUtils(base.BaseOSUtils):
         default_routes = [r for r in self._get_ipv4_routing_table()
                           if r[0] == '0.0.0.0']
         if default_routes:
-            return (default_routes[0][3], default_routes[0][2])
+            return default_routes[0][3], default_routes[0][2]
         else:
-            return (None, None)
+            return None, None
 
-    def _get_ipv4_routing_table(self):
-        routing_table = []
-
-        heap = kernel32.GetProcessHeap()
-
-        size = wintypes.ULONG(ctypes.sizeof(Win32_MIB_IPFORWARDTABLE))
-        p = kernel32.HeapAlloc(heap, 0, ctypes.c_size_t(size.value))
-        if not p:
+    @staticmethod
+    def _heap_alloc(heap, size):
+        table_mem = kernel32.HeapAlloc(heap, 0, ctypes.c_size_t(size.value))
+        if not table_mem:
             raise exception.CloudbaseInitException(
                 'Unable to allocate memory for the IP forward table')
+        return table_mem
+
+    @contextlib.contextmanager
+    def _get_forward_table(self):
+        heap = kernel32.GetProcessHeap()
+        forward_table_size = ctypes.sizeof(Win32_MIB_IPFORWARDTABLE)
+        size = wintypes.ULONG(forward_table_size)
+        table_mem = self._heap_alloc(heap, size)
+
         p_forward_table = ctypes.cast(
-            p, ctypes.POINTER(Win32_MIB_IPFORWARDTABLE))
+            table_mem, ctypes.POINTER(Win32_MIB_IPFORWARDTABLE))
 
         try:
             err = iphlpapi.GetIpForwardTable(p_forward_table,
                                              ctypes.byref(size), 0)
             if err == self.ERROR_INSUFFICIENT_BUFFER:
                 kernel32.HeapFree(heap, 0, p_forward_table)
-                p = kernel32.HeapAlloc(heap, 0, ctypes.c_size_t(size.value))
-                if not p:
-                    raise exception.CloudbaseInitException(
-                        'Unable to allocate memory for the IP forward table')
+                table_mem = self._heap_alloc(heap, size)
                 p_forward_table = ctypes.cast(
-                    p, ctypes.POINTER(Win32_MIB_IPFORWARDTABLE))
+                    table_mem,
+                    ctypes.POINTER(Win32_MIB_IPFORWARDTABLE))
+                err = iphlpapi.GetIpForwardTable(p_forward_table,
+                                                 ctypes.byref(size), 0)
 
-            err = iphlpapi.GetIpForwardTable(p_forward_table,
-                                             ctypes.byref(size), 0)
-            if err != self.ERROR_NO_DATA:
-                if err:
-                    raise exception.CloudbaseInitException(
-                        'Unable to get IP forward table. Error: %s' % err)
+            if err and err != kernel32.ERROR_NO_DATA:
+                raise exception.CloudbaseInitException(
+                    'Unable to get IP forward table. Error: %s' % err)
 
-                forward_table = p_forward_table.contents
-                table = ctypes.cast(
-                    ctypes.addressof(forward_table.table),
-                    ctypes.POINTER(Win32_MIB_IPFORWARDROW *
-                                   forward_table.dwNumEntries)).contents
-
-                i = 0
-                while i < forward_table.dwNumEntries:
-                    row = table[i]
-                    routing_table.append((
-                        encoding.get_as_string(Ws2_32.inet_ntoa(
-                            row.dwForwardDest)),
-                        encoding.get_as_string(Ws2_32.inet_ntoa(
-                            row.dwForwardMask)),
-                        encoding.get_as_string(Ws2_32.inet_ntoa(
-                            row.dwForwardNextHop)),
-                        row.dwForwardIfIndex,
-                        row.dwForwardMetric1))
-                    i += 1
-
-            return routing_table
+            yield p_forward_table
         finally:
             kernel32.HeapFree(heap, 0, p_forward_table)
 
+    def _get_ipv4_routing_table(self):
+        routing_table = []
+        with self._get_forward_table() as p_forward_table:
+            forward_table = p_forward_table.contents
+            table = ctypes.cast(
+                ctypes.addressof(forward_table.table),
+                ctypes.POINTER(Win32_MIB_IPFORWARDROW *
+                               forward_table.dwNumEntries)).contents
+
+            for row in table:
+                destination = Ws2_32.inet_ntoa(
+                    row.dwForwardDest).decode()
+                netmask = Ws2_32.inet_ntoa(
+                    row.dwForwardMask).decode()
+                gateway = Ws2_32.inet_ntoa(
+                    row.dwForwardNextHop).decode()
+                routing_table.append((
+                    destination,
+                    netmask,
+                    gateway,
+                    row.dwForwardIfIndex,
+                    row.dwForwardMetric1))
+
+        return routing_table
+
     def check_static_route_exists(self, destination):
         return len([r for r in self._get_ipv4_routing_table()
                     if r[0] == destination]) > 0
@@ -822,17 +825,14 @@ class WindowsUtils(base.BaseOSUtils):
                                                 VER_GREATER_EQUAL)
 
         type_mask = VER_MAJORVERSION | VER_MINORVERSION | VER_BUILDNUMBER
-        ret_val = kernel32.VerifyVersionInfoW(ctypes.byref(vi), type_mask,
-                                              mask)
-        if ret_val:
+        ret_val = ntdll.RtlVerifyVersionInfo(ctypes.byref(vi), type_mask, mask)
+        if not ret_val:
             return True
+        elif ret_val == self.STATUS_REVISION_MISMATCH:
+            return False
         else:
-            err = kernel32.GetLastError()
-            if err == self.ERROR_OLD_WIN_VERSION:
-                return False
-            else:
-                raise exception.CloudbaseInitException(
-                    "VerifyVersionInfo failed with error: %s" % err)
+            raise exception.CloudbaseInitException(
+                "RtlVerifyVersionInfo failed with error: %s" % ret_val)
 
     def get_volume_label(self, drive):
         max_label_size = 261
@@ -969,6 +969,33 @@ class WindowsUtils(base.BaseOSUtils):
 
         return physical_disks
 
+    def get_volumes(self):
+        """Retrieve a list with all the volumes found on all disks."""
+        volumes = []
+        volume = ctypes.create_unicode_buffer(chr(0) * self.MAX_PATH)
+
+        handle_volumes = kernel32.FindFirstVolumeW(volume, self.MAX_PATH)
+        if handle_volumes == self.INVALID_HANDLE_VALUE:
+            raise exception.WindowsCloudbaseInitException(
+                "FindFirstVolumeW failed: %r")
+
+        try:
+            while True:
+                volumes.append(volume.value)
+                found = kernel32.FindNextVolumeW(handle_volumes, volume,
+                                                 self.MAX_PATH)
+                if not found:
+                    errno = ctypes.GetLastError()
+                    if errno == self.ERROR_NO_MORE_FILES:
+                        break
+                    else:
+                        raise exception.WindowsCloudbaseInitException(
+                            "FindNextVolumeW failed: %r")
+        finally:
+            kernel32.FindVolumeClose(handle_volumes)
+
+        return volumes
+
     def _get_fw_protocol(self, protocol):
         if protocol == self.PROTOCOL_TCP:
             fw_protocol = self._FW_IP_PROTOCOL_TCP
@@ -1031,43 +1058,7 @@ class WindowsUtils(base.BaseOSUtils):
         C:\Windows\(System32|SysWOW64|Sysnative).
         Note that "Sysnative" is just an alias (doesn't really exist on disk).
 
-        On 32bit OSes, the return value will be the System32 directory,
-        which contains 32bit programs.
-        On 64bit OSes, the return value may be different, depending on the
-        Python bits and the `sysnative` parameter. If the Python interpreter is
-        32bit, the return value will be System32 (containing 32bit
-        programs) if `sysnative` is set to False and Sysnative otherwise. But
-        if the Python interpreter is 64bit and `sysnative` is False, the return
-        value will be SysWOW64 and System32 for a True value of `sysnative`.
-
-        Why this behavior and what is the purpose of `sysnative` parameter?
-
-        On a 32bit OS the things are clear, there is one System32 directory
-        containing 32bit applications and that's all. On a 64bit OS, there's a
-        System32 directory containing 64bit applications and a compatibility
-        one named SysWOW64 (WindowsOnWindows) containing the 32bit version of
-        them. Depending on the Python interpreter's bits, the `sysnative` flag
-        will try to bring the appropriate version of the system directory, more
-        exactly, the physical System32 or SysWOW64 found on disk. On a WOW case
-        (32bit interpreter on 64bit OS), a return value of System32 will point
-        to the physical SysWOW64 directory and a return value of Sysnative,
-        which is consolidated by the existence of this alias, will point to the
-        real physical System32 directory found on disk. If the OS is still
-        64bit and there is no WOW case (that means the interpreter is 64bit),
-        the system native concept is out of discussion and each return value
-        will point to the physical location it intends to.
-
-        On a 32bit OS the `sysnative` parameter has no meaning, but on a 64bit
-        one, based on its value, it will provide a real/alias path pointing to
-        system native applications if set to True (64bit programs) and to
-        system compatibility applications if set to False (32bit programs). Its
-        purpose is to provide the correct system paths by taking into account
-        the Python interpreter bits too, because on a 32bit interpreter
-        version, System32 is not the same with the System32 on a 64bit
-        interpreter. Also, using a 64bit interpreter, the Sysnative alias will
-        not work, but the `sysnative` parameter will take care to return
-        SysWOW64 if you explicitly want 32bit applications, by setting it to
-        False.
+        More info about this can be found in documentation.
         """
         if sysnative and self.check_sysnative_dir_exists():
             return self.get_sysnative_dir()
@@ -1075,14 +1066,33 @@ class WindowsUtils(base.BaseOSUtils):
             return self.get_syswow64_dir()
         return self.get_system32_dir()
 
+    def is_nano_server(self):
+        return self._check_server_level("NanoServer")
+
+    def _check_server_level(self, server_level):
+        try:
+            with winreg.OpenKey(
+                    winreg.HKEY_LOCAL_MACHINE,
+                    "Software\\Microsoft\\Windows NT\\CurrentVersion\\Server\\"
+                    "ServerLevels") as key:
+                return winreg.QueryValueEx(key, server_level)[0] == 1
+        except WindowsError as ex:
+            if ex.winerror == 2:
+                return False
+            else:
+                raise
+
     def execute_powershell_script(self, script_path, sysnative=True):
         base_dir = self._get_system_dir(sysnative)
         powershell_path = os.path.join(base_dir,
                                        'WindowsPowerShell\\v1.0\\'
                                        'powershell.exe')
 
-        args = [powershell_path, '-ExecutionPolicy', 'RemoteSigned',
-                '-NonInteractive', '-File', script_path]
+        args = [powershell_path]
+        if not self.is_nano_server():
+            args += ['-ExecutionPolicy', 'RemoteSigned', '-NonInteractive',
+                     '-File']
+        args.append(script_path)
 
         return self.execute_process(args, shell=False)
 
