@@ -22,6 +22,8 @@ PLUGINS = {
                    'cloudconfigplugins.write_files.WriteFilesPlugin',
     'set_timezone': 'cloudbaseinit.plugins.common.userdataplugins.'
                     'cloudconfigplugins.set_timezone.SetTimezonePlugin',
+    'set_hostname': 'cloudbaseinit.plugins.common.userdataplugins.'
+                    'cloudconfigplugins.set_hostname.SetHostnamePlugin',
 }
 
 
