@@ -12,6 +12,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import functools
 import socket
 import unittest
 
@@ -19,7 +20,7 @@ try:
     import unittest.mock as mock
 except ImportError:
     import mock
-from oslo.config import cfg
+from oslo_config import cfg
 from six.moves import urllib
 
 from cloudbaseinit.metadata.services import base
@@ -149,12 +150,17 @@ class CloudStackTest(unittest.TestCase):
 
     @mock.patch('cloudbaseinit.metadata.services.cloudstack.CloudStack'
                 '._get_cache_data')
-    def _test_cache_response(self, mock_get_cache_data, method, metadata):
+    def _test_cache_response(self, mock_get_cache_data, method, metadata,
+                             decode=True):
         mock_get_cache_data.side_effect = [mock.sentinel.response]
         response = method()
 
         self.assertEqual(mock.sentinel.response, response)
-        mock_get_cache_data.assert_called_once_with(metadata)
+        cache_assert = functools.partial(
+            mock_get_cache_data.assert_called_once_with,
+            metadata)
+        if decode:
+            cache_assert(decode=decode)
 
     def test_get_instance_id(self):
         self._test_cache_response(method=self._service.get_instance_id,
@@ -166,7 +172,7 @@ class CloudStackTest(unittest.TestCase):
 
     def test_get_user_data(self):
         self._test_cache_response(method=self._service.get_user_data,
-                                  metadata='../user-data')
+                                  metadata='../user-data', decode=False)
 
     @mock.patch('cloudbaseinit.metadata.services.cloudstack.CloudStack'
                 '._get_cache_data')
@@ -270,10 +276,26 @@ class CloudStackTest(unittest.TestCase):
         mock_response.read.side_effect = [cloudstack.BAD_REQUEST,
                                           cloudstack.SAVED_PASSWORD]
         mock_response.status = 400
-        self.assertIsNone(self._service._delete_password())
-        mock_response.status = 200
-        self.assertIsNone(self._service._delete_password())
-        self.assertEqual(2, mock_request.call_count)
+        expected_output = [
+            'Remove the password for this instance from the '
+            'Password Server.',
+            'Removing password failed',
+            'Fail to remove the password from the Password Server.',
+
+            'Remove the password for this instance from the '
+            'Password Server.',
+            'The password was removed from the Password Server',
+
+        ]
+
+        with testutils.LogSnatcher('cloudbaseinit.metadata.services.'
+                                   'cloudstack') as snatcher:
+            self.assertIsNone(self._service._delete_password())
+            mock_response.status = 200
+            self.assertIsNone(self._service._delete_password())
+            self.assertEqual(2, mock_request.call_count)
+        for expected, output in zip(expected_output, snatcher.output):
+            self.assertTrue(output.startswith(expected))
 
     @mock.patch('cloudbaseinit.metadata.services.cloudstack.CloudStack.'
                 '_delete_password')
